<?php

declare(strict_types=1);

namespace Drupal\rokka\Plugin\ImageToolkit;

use Drupal\system\Plugin\ImageToolkit\GDToolkit;

/**
 * Provides ImageMagick integration toolkit for image manipulation.
 *
 * @ImageToolkit(
 *   id = "rokka",
 *   title = @Translation("Rokka image toolkit")
 * )
 */
class RokkaToolkit extends GDToolkit {

  /**
   * {@inheritdoc}
   */
  public function isValid() {
    // We assume, that all sucessfully uploaded images are valid.
    if (!empty($this->getSource()) && strpos($this->getSource(), 'rokka://') === FALSE) {
      // Case happens during upload to /tmp folder before the file is transferred to rokka.io.
      return ((bool) $this->getMimeType());
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getHeight() {
    if (!empty($this->getSource()) && strpos($this->getSource(), 'rokka://') === 0) {
      $data = NULL;
      // Use the cached meta data from the rokka metadata table.
      $metadata = \Drupal::service('rokka.service')->loadRokkaMetadataByUri($this->getSource());
      if (!empty($metadata)) {
        $metadata = reset($metadata);
        return $metadata->getHeight();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getWidth() {
    if (!empty($this->getSource()) && strpos($this->getSource(), 'rokka://') === 0) {
      $data = NULL;
      // Use the cached meta data from the rokka metadata table.
      $metadata = \Drupal::service('rokka.service')->loadRokkaMetadataByUri($this->getSource());
      if (!empty($metadata)) {
        $metadata = reset($metadata);
        return $metadata->getWidth();
      }
    }
  }

  public function getMimeType() {
    $mime_type = $this->getType() ? image_type_to_mime_type($this->getType()) : '';

    // GD library does not support SVG, but rokka does. So we will trick the toolkit
    // and detect svg as a mime time the first time the file is uploaded to tmp folder.
    // If we detect an svg image, we allow the upload.
    if (empty($mime_type)) {
      if (!empty($this->getSource()) && strpos($this->getSource(), 'rokka://') === FALSE) {
        $rokka_additional_allowed_mime_types = ['image/svg', 'image/svg+xml'];
        $mime_type_guessed = mime_content_type($this->getSource());
        if (in_array($mime_type_guessed, $rokka_additional_allowed_mime_types)) {
          $mime_type = $mime_type_guessed;
        }
      }
    }

    return $mime_type;
  }

  /**
   * {@inheritdoc}
   */
  public function parseFile() {

    $source = $this->getSource();
    if (!empty($source) && strpos($source, 'rokka://') === 0) {
      $data = NULL;
      // Use the cached meta data from the rokka metadata table.
      $metadata = \Drupal::service('rokka.service')->loadRokkaMetadataByUri($this->getSource());
      if (!empty($metadata)) {
        $metadata = reset($metadata);

        // Mimic the getimagesize function of php.
        $data[0] = $metadata->getWidth();
        $data[1] = $metadata->getHeight();
        $data[2] = $this->extensionToImageType($metadata->getFormat());
      }

      if ($data && in_array($data[2], static::supportedTypes(), TRUE)) {
        $this->setType($data[2]);
        $this->preLoadInfo = $data;
        return TRUE;
      }
      return FALSE;
    }
    else {
      // Case happens during upload to /tmp folder before the file is transferred to rokka.io.
      return parent::parseFile();
    }
  }

}
